# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>


###############################################################################
# Release log
###############################################################################
# 17/10/2010
# Version Alpha 2.2.01 20101017 released
#
# 29/12/2010
# Version Alpha 2.2.02 20101229 released
# Mirror property were used to set the specular shader instead of transparency
# Roughness map exported based on hardness mapping
# Several fixes in material conversion
# Other objects types are now exported (curve, text, meta, ...) 
# Check if camera is valid and selected when export camera is activated
# Check if sun is valid and selected when export sun is activated
# Set the "samples per image" according to the "presets samples per image" dropbox selection
# Take all the advanced options into consideration for obj exporting
# Fix issue with material assignment for curve and text objects 
# Now it should export duplicates as well as object and group particules
###############################################################################

bl_addon_info = {
    "name": "Octane",
    "author": "EnricoErica",
    "version": (5,1),
    "blender": (2, 5, 6),
    "api": 33713,
    "location": "Render > Engine > Octane",
    "description": "Exports and launch render in Octane",
    "warning": "Beta version - Not originial script from Enricoerica - use at own risk",
    "wiki_url": "http://www.refractivesoftware.com/index.php",
    "tracker_url": "",
    "category": "Render"}


from math import atan, pi, degrees
import subprocess
import os
import sys
import time
import mathutils
import bpy

ostype = sys.platform

def matrix_vect(mat,vec):
	vecr = [0.,0.,0.,0.,0.]
	for i in range(4):
		for j in range(4):
			vecr[i] += vec[j] * mat[j][i] 
	return vecr
		
def rotate90x(vect):
	return [vect[0],vect[2],-vect[1]]
	
def fixName(name):
	if name == None:
		return 'None'
	else:
		return name.replace(' ', '_')

def fixPath(path):
	path=str(path)

	if ostype == "linux2" or ostype == "darwin":
		path=path.replace("\\","/")
	if ostype == "win32":
		path=path.replace("/","\\")

	return path

def copy_file(source, dest):
	file = open(source, 'rb')
	data = file.read()
	file.close()

	file = open(dest, 'wb')
	file.write(data)
	file.close()

def write_mtl(scene, filepath, copy_images, mtl_dict):

    world = scene.world
    worldAmb = world.ambient_color

    dest_dir = os.path.dirname(filepath)

    def copy_image(image):
        fn = image.filepath
        fn_strip = os.path.basename(fn)
        print (fn_strip)

        if copy_images:
            rel = fn_strip
            fn_abs_dest = os.path.join(dest_dir, fn_strip)
            if not os.path.exists(fn_abs_dest):
                copy_file(fn, fn_abs_dest)
        else:
            rel = fn

        return rel


    file = open(filepath, "w")
    # XXX
#       file.write('# Blender MTL File: %s\n' % Blender.Get('filepath').split('\\')[-1].split('/')[-1])
    file.write('# Material Count: %i\n' % len(mtl_dict))
    # Write material/image combinations we have used.
    for key, (mtl_mat_name, mat, img) in mtl_dict.items():

        # Get the Blender data for the material and the image.
        # Having an image named None will make a bug, dont do it :)

        file.write('newmtl %s\n' % mtl_mat_name) # Define a new material: matname_imgname

        if mat:
              #file.write('Ns %.6f\n' % (mat.specular_hardness*10) ) # Hardness, convert blenders 1-511 to MTL's
			  
              if (mat.use_transparency and mat.transparency_method=="RAYTRACE"):
                #file.write('Ns %.6f\n' % ((1.0-mat.raytrace_transparency.gloss_factor)*10+0.01)) # Roughness
                file.write('Kd %.6f %.6f %.6f\n' % tuple([c for c in mat.diffuse_color]) ) # Diffuse
                file.write('Ka %.6f %.6f %.6f\n' %  tuple([c*mat.ambient for c in worldAmb])  ) # Ambient, uses mirror colour,
                file.write('Ns %.6f\n' % ((mat.specular_hardness-1) * 1.9607843137254901)) # Glossiness - Hardness, convert blenders 1-511 to MTL's
                file.write('Ks %.6f %.6f %.6f\n' % tuple([c for c in mat.specular_color]) ) # Specular
                file.write('Ni %.6f\n' % mat.raytrace_transparency.ior) # Refraction index
                
              elif (mat.raytrace_mirror.use):
                #file.write('Ns %.6f\n' % ((1.0-mat.raytrace_mirror.gloss_factor)*10+0.01)) # Roughness
                file.write('Kd %.6f %.6f %.6f\n' % tuple([c for c in mat.diffuse_color]) ) # Diffuse
                file.write('Ka %.6f %.6f %.6f\n' %  tuple([c*mat.specular_intensity for c in mat.mirror_color])  ) # Ambient, uses mirror colour,
                file.write('Ns %.6f\n' % ((mat.specular_hardness-1) * 1.9607843137254901)) # Glossiness - Hardness, convert blenders 1-511 to MTL's
                file.write('Ks %.6f %.6f %.6f\n' % tuple([c for c in mat.specular_color]) ) # Specular
                file.write('Ni %.6f\n' % (1+(mat.raytrace_mirror.reflect_factor*7))) # Refraction index
                
              else:
                file.write('Kd %.6f %.6f %.6f\n' % tuple([c for c in mat.diffuse_color]) ) # Diffuse
                file.write('Ns %.6f\n' % ((mat.specular_hardness-1) * 1.9607843137254901)) # Glossiness - Hardness, convert blenders 1-511 to MTL's
                file.write('Ks %.6f %.6f %.6f\n' % tuple([c for c in mat.specular_color]) ) # Specular
              

              
              file.write('d %.6f\n' % mat.alpha) # Alpha (obj uses 'd' for dissolve)
              # 0 to disable lighting, 1 for ambient & diffuse only (specular color set to black), 2 for full lighting.
              if mat.use_shadeless:
                      file.write('illum 0\n') # ignore lighting
              elif mat.specular_intensity == 0:
                      file.write('illum 1\n') # no specular.
              else:
                      file.write('illum 2\n') # light normaly

        else:
            #write a dummy material here?
            file.write('Ns 0\n')
            file.write('Ka %.6f %.6f %.6f\n' %  tuple([c for c in worldAmb])  ) # Ambient, uses mirror colour,
            file.write('Kd 0.8 0.8 0.8\n')
            file.write('Ks 0.8 0.8 0.8\n')
            file.write('d 1\n') # No alpha
            file.write('illum 2\n') # light normaly


        if mat: # No face image. if we havea material search for MTex image.
            for mtex in mat.texture_slots:
                if mtex and mtex.texture.type == 'IMAGE':
                    try:
                        #filename = mtex.texture.image.filepath.split('\\')[-1].split('/')[-1]
                        print (mtex.texture.image.filepath)
                        filename=copy_image(mtex.texture.image)

                        if mtex.use_map_color_diffuse:
                            file.write('map_Kd %s\n' % filename) # Diffuse mapping image
						
                        if mtex.use_map_normal:
                            file.write('map_bump %s\n' % filename) # Bump mapping image
						
                        if mtex.use_map_specular:
                            file.write('map_Ks %s\n' % filename) # Specular mapping image
						
                        if mtex.use_map_alpha:
                            file.write('map_d %s\n' % filename) # Alpha mapping image
						
                        if mtex.use_map_mirror:
                            file.write('map_Ka %s\n' % filename) # Mirror mapping image

                        if mtex.use_map_hardness:
                            file.write('map_Ns %s\n' % filename) # Hardness to affect roughness in octane
                    except:
                        # Texture has no image though its an image type, best ignore.
                        print ("No image")


        file.write('\n\n')

    file.close()
    
def test_nurbs_compat(ob):
    if ob.type != 'Curve':
        return False

    for nu in ob.data:
        if (not nu.knotsV) and nu.type != 1: # not a surface and not bezier
            return True

    return False


# XXX not converted
def write_nurb(file, ob, ob_mat):
    tot_verts = 0
    cu = ob.data

    # use negative indices
    Vector = Blender.mathutils.Vector
    for nu in cu:

        if nu.type==0:          DEG_ORDER_U = 1
        else:                           DEG_ORDER_U = nu.orderU-1  # Tested to be correct

        if nu.type==1:
            print("\tWarning, bezier curve:", ob.name, "only poly and nurbs curves supported")
            continue

        if nu.knotsV:
            print("\tWarning, surface:", ob.name, "only poly and nurbs curves supported")
            continue

        if len(nu) <= DEG_ORDER_U:
            print("\tWarning, orderU is lower then vert count, skipping:", ob.name)
            continue

        pt_num = 0
        do_closed = (nu.flagU & 1)
        do_endpoints = (do_closed==0) and (nu.flagU & 2)

        for pt in nu:
            pt = Vector(pt[0], pt[1], pt[2]) * ob_mat
            file.write('v %.6f %.6f %.6f\n' % (pt[0], pt[1], pt[2]))
            pt_num += 1
        tot_verts += pt_num

        file.write('g %s\n' % (fixName(ob.name))) # fixName(ob.getData(1)) could use the data name too
        file.write('cstype bspline\n') # not ideal, hard coded
        file.write('deg %d\n' % DEG_ORDER_U) # not used for curves but most files have it still

        curve_ls = [-(i+1) for i in range(pt_num)]

        # 'curv' keyword
        if do_closed:
            if DEG_ORDER_U == 1:
                pt_num += 1
                curve_ls.append(-1)
            else:
                pt_num += DEG_ORDER_U
                curve_ls = curve_ls + curve_ls[0:DEG_ORDER_U]

        file.write('curv 0.0 1.0 %s\n' % (' '.join( [str(i) for i in curve_ls] ))) # Blender has no U and V values for the curve

        # 'parm' keyword
        tot_parm = (DEG_ORDER_U + 1) + pt_num
        tot_parm_div = float(tot_parm-1)
        parm_ls = [(i/tot_parm_div) for i in range(tot_parm)]

        if do_endpoints: # end points, force param
            for i in range(DEG_ORDER_U+1):
                parm_ls[i] = 0.0
                parm_ls[-(1+i)] = 1.0

        file.write('parm u %s\n' % ' '.join( [str(i) for i in parm_ls] ))

        file.write('end\n')

    return tot_verts

def write(filepath, objects, scene,
          EXPORT_TRI=False,
          EXPORT_EDGES=False,
          EXPORT_NORMALS=False,
          EXPORT_NORMALS_HQ=False,
          EXPORT_UV=True,
          EXPORT_MTL=True,
          EXPORT_COPY_IMAGES=False,
          EXPORT_APPLY_MODIFIERS=True,
          EXPORT_ROTX90=True,
          EXPORT_BLEN_OBS=True,
          EXPORT_GROUP_BY_OB=False,
          EXPORT_GROUP_BY_MAT=False,
          EXPORT_KEEP_VERT_ORDER=False,
          EXPORT_POLYGROUPS=False,
          EXPORT_CURVE_AS_NURBS=True):
    '''
    Basic write function. The context and options must be alredy set
    This can be accessed externaly
    eg.
    write( 'c:\\test\\foobar.obj', Blender.Object.GetSelected() ) # Using default options.
    '''

    # XXX
    import math

    unitFactor = 1
	#Set the unit factor
    unitFactor = {
        0:0.001,
        1:0.01,
        2:0.1,
        3:1,
        4:10,
        5:100,
        6:1000,
        7:0.0254,
        8:0.3048,
        9:0.9144,
        10:201.168,
        11:1609.344               
    }[int(scene.oct_unit_size)]

    def veckey3d(v):
        return round(v.x, 6), round(v.y, 6), round(v.z, 6)

    def veckey2d(v):
        return round(v[0], 6), round(v[1], 6)
        # return round(v.x, 6), round(v.y, 6)

    def findVertexGroupName(face, vWeightMap):
        """
        Searches the vertexDict to see what groups is assigned to a given face.
        We use a frequency system in order to sort out the name because a given vetex can
        belong to two or more groups at the same time. To find the right name for the face
        we list all the possible vertex group names with their frequency and then sort by
        frequency in descend order. The top element is the one shared by the highest number
        of vertices is the face's group
        """
        weightDict = {}
        for vert_index in face.vertices:
            vWeights = vWeightMap[vert_index]
            for vGroupName, weight in vWeights:
                weightDict[vGroupName] = weightDict.get(vGroupName, 0) + weight

        if weightDict:
            alist = [(weight,vGroupName) for vGroupName, weight in weightDict.items()] # sort least to greatest amount of weight
            alist.sort()
            return(alist[-1][1]) # highest value last
        else:
            return '(null)'

    # TODO: implement this in C? dunno how it should be called...
    def getVertsFromGroup(me, group_index):
        ret = []

        for i, v in enumerate(me.vertices):
            for g in v.groups:
                if g.group == group_index:
                    ret.append((i, g.weight))

        return ret


    print('OBJ Export path: "%s"' % filepath)
    temp_mesh_name = '~tmp-mesh'
    time1 = time.clock()

    file = open(filepath, "w")
    # Write Header
    file.write('# Blender v%s OBJ File: %s\n' % (bpy.app.version_string, bpy.data.filepath.split('/')[-1].split('\\')[-1] ))
    file.write('# www.blender.org\n')

    # Tell the obj file what material file to use.
    if EXPORT_MTL:
        mtlfilepath = '%s.mtl' % '.'.join(filepath.split('.')[:-1])
        file.write('mtllib %s\n' % ( mtlfilepath.split('\\')[-1].split('/')[-1] ))

    if EXPORT_ROTX90:
        mat_xrot90= mathutils.Matrix.Rotation(-math.pi/2, 4, 'X')

    # Initialize totals, these are updated each object
    totverts = totuvco = totno = 1

    face_vert_index = 1

    globalNormals = {}

    # A Dict of Materials
    # (material.name, image.name):matname_imagename # matname_imagename has gaps removed.
    mtl_dict = {}

    # Get all meshes
    for ob_main in objects:
#        if ob_main.parent and ob_main.parent.duplis_used:
#          print(ob_main.type, 'duplis ')
        #,ob_main.parent.dupli_type)
        # ignore dupli children
        if ob_main.parent and ob_main.parent.dupli_type != 'NONE':
            # XXX
            print(ob_main.name, 'is a dupli child - ignoring')
            continue

        obs = []
        if ob_main.dupli_type != 'NONE':
            # XXX
            print('creating dupli_list on', ob_main.name)
            ob_main.create_dupli_list(scene)

            obs = [(dob.object, dob.matrix) for dob in ob_main.dupli_list]

            # XXX debug print
            print(ob_main.name, 'has', len(obs), 'dupli children')
        else:
            obs = [(ob_main, ob_main.matrix_world)]

        for ob, ob_mat in obs:

            # Nurbs curve support
            if EXPORT_CURVE_AS_NURBS and test_nurbs_compat(ob):
                if EXPORT_ROTX90:
                   ob_mat = ob_mat * mat_xrot90
                totverts += write_nurb(file, ob, ob_mat)
                continue
            # END NURBS

#             if ob.type != 'MESH':
#                 continue

            me = ob.create_mesh(scene, EXPORT_APPLY_MODIFIERS, 'RENDER')
#             me = ob.create_mesh(scene, EXPORT_APPLY_MODIFIERS, 'PREVIEW')

#                       # Will work for non meshes now! :)
#            me= BPyMesh.getMeshFromObject(ob, none, True, True, scene)
            if not me:
                continue

            if EXPORT_ROTX90:
                me.transform(mat_xrot90 * ob_mat)
            else:
                me.transform(ob_mat)


            if EXPORT_UV:
                faceuv = len(me.uv_textures) > 0
                if faceuv:
                    uv_layer = me.uv_textures.active.data[:]
            else:
                faceuv = False

            me_verts = me.vertices[:]

            # Make our own list so it can be sorted to reduce context switching
            face_index_pairs = [ (face, index) for index, face in enumerate(me.faces)]
            # faces = [ f for f in me.faces ]

            if EXPORT_EDGES:
                edges = me.edges
            else:
                edges = []

            if not (len(face_index_pairs)+len(edges)+len(me.vertices)): # Make sure there is somthing to write

                # clean up
                bpy.data.meshes.remove(me)

                continue # dont bother with this mesh.

            # XXX
            # High Quality Normals
            if EXPORT_NORMALS and face_index_pairs:
                me.calc_normals()
#                               if EXPORT_NORMALS_HQ:
#                                       BPyMesh.meshCalcNormals(me)
#                               else:
#                                       # transforming normals is incorrect
#                                       # when the matrix is scaled,
#                                       # better to recalculate them
#                                       me.calcNormals()

			#materials = me.getMaterials(1) # 1 == will return None in the list.
            if ob.type == "TEXT" or ob.type == "CURVE":
                materials = ob.data.materials
            else:
                materials = me.materials

            materialNames = []
            materialItems = [m for m in materials]
            if materials:
                for mat in materials:
                    if mat: # !=None
                        materialNames.append(mat.name)
                    else:
                        materialNames.append(None)
                # Cant use LC because some materials are None.
                # materialNames = map(lambda mat: mat.name, materials) # Bug Blender, dosent account for null materials, still broken.

            # Possible there null materials, will mess up indicies
            # but at least it will export, wait until Blender gets fixed.
            materialNames.extend((16-len(materialNames)) * [None])
            materialItems.extend((16-len(materialItems)) * [None])

            # Sort by Material, then images
            # so we dont over context switch in the obj file.
            if EXPORT_KEEP_VERT_ORDER:
                pass
            elif faceuv:
                # XXX update
                tface = me.uv_textures.active.data

                face_index_pairs.sort(key=lambda a: (a[0].material_index, hash(tface[a[1]].image), a[0].use_smooth))
            elif len(materials) > 1:
                face_index_pairs.sort(key = lambda a: (a[0].material_index, a[0].use_smooth))
            else:
                # no materials
                face_index_pairs.sort(key = lambda a: a[0].use_smooth)
#                       if EXPORT_KEEP_VERT_ORDER:
#                               pass
#                       elif faceuv:
#                               try:    faces.sort(key = lambda a: (a.mat, a.image, a.use_smooth))
#                               except: faces.sort(lambda a,b: cmp((a.mat, a.image, a.use_smooth), (b.mat, b.image, b.use_smooth)))
#                       elif len(materials) > 1:
#                               try:    faces.sort(key = lambda a: (a.mat, a.use_smooth))
#                               except: faces.sort(lambda a,b: cmp((a.mat, a.use_smooth), (b.mat, b.use_smooth)))
#                       else:
#                               # no materials
#                               try:    faces.sort(key = lambda a: a.use_smooth)
#                               except: faces.sort(lambda a,b: cmp(a.use_smooth, b.use_smooth))

            faces = [pair[0] for pair in face_index_pairs]

            # Set the default mat to no material and no image.
            contextMat = (0, 0) # Can never be this, so we will label a new material teh first chance we get.
            contextSmooth = None # Will either be true or false,  set bad to force initialization switch.

            if EXPORT_BLEN_OBS or EXPORT_GROUP_BY_OB:
                name1 = ob.name
                name2 = ob.data.name
                if name1 == name2:
                    obnamestring = fixName(name1)
                else:
                    obnamestring = '%s_%s' % (fixName(name1), fixName(name2))

                if EXPORT_BLEN_OBS:
                    file.write('o %s\n' % obnamestring) # Write Object name
                else: # if EXPORT_GROUP_BY_OB:
                    file.write('g %s\n' % obnamestring)


            # Vert
            for v in me.vertices:
                file.write('v %.6f %.6f %.6f\n' % tuple(v.co*unitFactor))

            # UV
            if faceuv:
                uv_face_mapping = [[0,0,0,0] for i in range(len(face_index_pairs))] # a bit of a waste for tri's :/

                uv_dict = {} # could use a set() here
                uv_layer = me.uv_textures.active.data
                for f, f_index in face_index_pairs:
                    for uv_index, uv in enumerate(uv_layer[f_index].uv):
                        uvkey = veckey2d(uv)
                        try:
                            uv_face_mapping[f_index][uv_index] = uv_dict[uvkey]
                        except:
                            uv_face_mapping[f_index][uv_index] = uv_dict[uvkey] = len(uv_dict)
                            file.write('vt %.6f %.6f\n' % tuple(uv))

                uv_unique_count = len(uv_dict)
#               del uv, uvkey, uv_dict, f_index, uv_index
                # Only need uv_unique_count and uv_face_mapping

            # NORMAL, Smooth/Non smoothed.
            if EXPORT_NORMALS:
                for f, f_index in face_index_pairs:
                    if f.use_smooth:
                        for v_idx in f.vertices:
                            v = me_verts[v_idx]
                            noKey = veckey3d(v.normal)
                            if noKey not in globalNormals:
                                globalNormals[noKey] = totno
                                totno +=1
                                file.write('vn %.6f %.6f %.6f\n' % noKey)
                    else:
                        # Hard, 1 normal from the face.
                        noKey = veckey3d(f.normal)
                        if noKey not in globalNormals:
                            globalNormals[noKey] = totno
                            totno +=1
                            file.write('vn %.6f %.6f %.6f\n' % noKey)

            if not faceuv:
                f_image = None

            # XXX
            if EXPORT_POLYGROUPS:
                # Retrieve the list of vertex groups
#                               vertGroupNames = me.getVertGroupNames()

                currentVGroup = ''
                # Create a dictionary keyed by face id and listing, for each vertex, the vertex groups it belongs to
                vgroupsMap = [[] for _i in range(len(me.vertices))]
#                               vgroupsMap = [[] for _i in xrange(len(me.verts))]
                for g in ob.vertex_groups:
#                               for vertexGroupName in vertGroupNames:
                    for vIdx, vWeight in getVertsFromGroup(me, g.index):
#                                       for vIdx, vWeight in me.getVertsFromGroup(vertexGroupName, 1):
                        vgroupsMap[vIdx].append((g.name, vWeight))

            for f_index, f in enumerate(faces):
                f_v = [{"index": index, "vertex": me.vertices[index]} for index in f.vertices]

                # if f.verts[3] == 0:
                #       f_v.pop()

#                               f_v= f.v
                f_smooth= f.use_smooth
                f_mat = min(f.material_index, len(materialNames)-1)
#                               f_mat = min(f.mat, len(materialNames)-1)
                if faceuv:

                    tface = me.uv_textures.active.data[face_index_pairs[f_index][1]]

                    f_image = tface.image
                    f_uv = tface.uv
                    # f_uv= [tface.uv1, tface.uv2, tface.uv3]
                    # if len(f.verts) == 4:
                    #   f_uv.append(tface.uv4)
#                                       f_image = f.image
#                                       f_uv= f.uv

                # MAKE KEY
                if faceuv and f_image: # Object is always true.
                    key = materialNames[f_mat],  f_image.name
                else:
                    key = materialNames[f_mat],  None # No image, use None instead.

                # Write the vertex group
                if EXPORT_POLYGROUPS:
                    if len(ob.vertex_groups):
                        # find what vertext group the face belongs to
                        theVGroup = findVertexGroupName(f,vgroupsMap)
                        if      theVGroup != currentVGroup:
                            currentVGroup = theVGroup
                            file.write('g %s\n' % theVGroup)
#                               # Write the vertex group
#                               if EXPORT_POLYGROUPS:
#                                       if vertGroupNames:
#                                               # find what vertext group the face belongs to
#                                               theVGroup = findVertexGroupName(f,vgroupsMap)
#                                               if      theVGroup != currentVGroup:
#                                                       currentVGroup = theVGroup
#                                                       file.write('g %s\n' % theVGroup)

                # CHECK FOR CONTEXT SWITCH
                if key == contextMat:
                    pass # Context alredy switched, dont do anything
                else:
                    if key[0] == None and key[1] == None:
                        # Write a null material, since we know the context has changed.
                        if EXPORT_GROUP_BY_MAT:
                            # can be mat_image or (null)
                            file.write('g %s_%s\n' % (fixName(ob.name), fixName(ob.data.name)) ) # can be mat_image or (null)
                        file.write('usemtl (null)\n') # mat, image

                    else:
                        mat_data= mtl_dict.get(key)
                        if not mat_data:
                            # First add to global dict so we can export to mtl
                            # Then write mtl

                            # Make a new names from the mat and image name,
                            # converting any spaces to underscores with fixName.

                            # If none image dont bother adding it to the name
#                            if key[1] == None:
                                mat_data = mtl_dict[key] = ('%s'%fixName(key[0])), materialItems[f_mat], f_image
 #                           else:
  #                              mat_data = mtl_dict[key] = ('%s_%s' % (fixName(key[0]), fixName(key[1]))), materialItems[f_mat], f_image

                        if EXPORT_GROUP_BY_MAT:
                            file.write('g %s_%s_%s\n' % (fixName(ob.name), fixName(ob.data.name), mat_data[0]) ) # can be mat_image or (null)

                        file.write('usemtl %s\n' % mat_data[0]) # can be mat_image or (null)

                contextMat = key
                if f_smooth != contextSmooth:
                    if f_smooth: # on now off
                        file.write('s 1\n')
                        contextSmooth = f_smooth
                    else: # was off now on
                        file.write('s off\n')
                        contextSmooth = f_smooth

                file.write('f')
                if faceuv:
                    if EXPORT_NORMALS:
                        if f_smooth: # Smoothed, use vertex normals
                            for vi, v in enumerate(f_v):
                                file.write( ' %d/%d/%d' % \
                                                (v["index"] + totverts,
                                                 totuvco + uv_face_mapping[f_index][vi],
                                                 globalNormals[ veckey3d(v["vertex"].normal) ]) ) # vert, uv, normal

                        else: # No smoothing, face normals
                            no = globalNormals[ veckey3d(f.normal) ]
                            for vi, v in enumerate(f_v):
                                file.write( ' %d/%d/%d' % \
                                                (v["index"] + totverts,
                                                 totuvco + uv_face_mapping[f_index][vi],
                                                 no) ) # vert, uv, normal
                    else: # No Normals
                        for vi, v in enumerate(f_v):
                            file.write( ' %d/%d' % (\
                              v["index"] + totverts,\
                              totuvco + uv_face_mapping[f_index][vi])) # vert, uv

                    face_vert_index += len(f_v)

                else: # No UV's
                    if EXPORT_NORMALS:
                        if f_smooth: # Smoothed, use vertex normals
                            for v in f_v:
                                file.write( ' %d//%d' %
                                            (v["index"] + totverts, globalNormals[ veckey3d(v["vertex"].normal) ]) )
                        else: # No smoothing, face normals
                            no = globalNormals[ veckey3d(f.normal) ]
                            for v in f_v:
                                file.write( ' %d//%d' % (v["index"] + totverts, no) )
                    else: # No Normals
                        for v in f_v:
                            file.write( ' %d' % (v["index"] + totverts) )

                file.write('\n')

            # Write edges.
            if EXPORT_EDGES:
                for ed in edges:
                    if ed.is_loose:
                        file.write('f %d %d\n' % (ed.vertices[0] + totverts, ed.vertices[1] + totverts))

            # Make the indicies global rather then per mesh
            totverts += len(me.vertices)
            if faceuv:
                totuvco += uv_unique_count

            # clean up
            bpy.data.meshes.remove(me)

        if ob_main.dupli_type != 'NONE':
            ob_main.free_dupli_list()

    file.close()


    # Now we have all our materials, save them
    if EXPORT_MTL:
        write_mtl(scene, mtlfilepath, scene.oct_export_copy_images, mtl_dict)
#       if EXPORT_COPY_IMAGES:
#               dest_dir = os.path.basename(filepath)
# #             dest_dir = filepath
# #             # Remove chars until we are just the path.
# #             while dest_dir and dest_dir[-1] not in '\\/':
# #                     dest_dir = dest_dir[:-1]
#               if dest_dir:
#                       copy_images(dest_dir, mtl_dict)
#               else:
#                       print('\tError: "%s" could not be used as a base for an image path.' % filepath)

    print("OBJ Export time: %.2f" % (time.clock() - time1))
#    print "OBJ Export time: %.2f" % (sys.time() - time1)


#end obj part

# Octane Panel
from bpy.props import *

scene=bpy.types.Scene
 
# Octane project configuration
scene.oct_render_binary=StringProperty(name="Octane Binary",
				description="Octane render binary",
				maxlen = 128,
# 				default = default_path,
				default = "",
				subtype='FILE_PATH')

scene.oct_project_path=StringProperty(name="Project path",
				description="Octane project path",
				maxlen = 128,
#				default = default_path,
				default = "",
				subtype='FILE_PATH')

scene.oct_project_name=StringProperty(name="Project name",
				description="Octane project name",
				maxlen = 128,
#				 default = default_path,
				default = "default",
				subtype='FILE_PATH')

scene.oct_replace_project=BoolProperty(name="Replace",
				description="Force the replacement of the project file",
				default = False)

scene.oct_unit_size=EnumProperty(  
				name="Native Unit Size",
				description="Native Unit Size",
				items=(
					("11",  "Miles", ""),
					("10", "Furlongs", ""),
					("9",   "Yards", ""),
					("8",   "Feets", ""),
					("7",   "Inches", ""),
					("6",   "Kilometers", ""),
					("5",   "Hectometers", ""),
					("4",   "Decameters", ""),
					("3",   "Meters", ""),
					("2",   "Decimeters", ""),
					("1",   "Centimeters", ""),
					("0",   "Milimeters", "")
				),
				default= "3")

# Octane Export configuration
# Film
scene.oct_resolution=BoolProperty(
				name="Resolution",
				description="Use personal resolution",
				default = False)

scene.oct_resolution_width=IntProperty(attr="",
				name="Width",
				description="Width",
				min=1, max=4096, default=1024)

scene.oct_resolution_height=IntProperty(attr="",
				name="Height",
				description="Height",
				min=1, max=4096, default=512)

scene.oct_percent_size=EnumProperty(  
				name="",
				description="Percent size",
				items=(
					("title","Unit", ""),
					("0",  "100%", ""),
					("1", "75%", ""),
					("2",   "50%", ""),
					("3",   "25%", "")
				),
				default= "0")

# Camera

scene.oct_export_camera=BoolProperty(
				name="Export Camera",
				description="Export Camera",
				default = True)

scene.oct_active_camera=StringProperty(
				name="Active Camera",
				description="Active Camera",
				maxlen = 128,
#				 default = default_path,
				default = "")

scene.oct_lens_aperture=BoolProperty(
				name="Lens Aperture",
				description="Lens Aperture",
				default = False)

scene.oct_lens_aperture_value=IntProperty(attr="",
				name="Value",
				description="Value",
				min=-1, max=100, default=1)

scene.oct_lens_aperture_curve=BoolProperty(
				name="Curve",
				description="Curve - Not implemented yet",
				default = False)

scene.oct_focal_depth=BoolProperty(
				name="Focal Depth",
				description="Focal Depth",
				default = False)

scene.oct_focal_depth_value=IntProperty(attr="",
				name="Value",
				description="Value",
				min=-1, max=100, default=5)

scene.oct_focal_depth_curve=BoolProperty(
				name="Curve",
				description="Curve - Not implemented yet",
				default = False)

scene.oct_camera_motion=BoolProperty(
				name="Camera Motion",
				description="Use camera motion for animation",
				default = False)

scene.oct_interpolate_frame=EnumProperty(  
				name="Interpolate",
				description="Interpolate frame",
				items=(
					("0",  "Next", ""),
					("1", "Previous", "")
				),
				default= "0")

# Day light environment

scene.oct_export_sundir=BoolProperty(
				name="Export Sun Dir",
				description="Export Sun Dir",
				default = True)

scene.oct_active_light_source=StringProperty(
				name="Active Light Source",
				description="Active Light Source",
				maxlen = 128,
#				 default = default_path,
				default = "")

# GPU Device

scene.oct_GPU_selector=BoolProperty(
				name="GPU Devices to use",
				description="Specify GPU devices to use",
				default = False)

scene.oct_GPU_use_list=StringProperty(
				name="",
				description="GPUs to use (ex: 0 1 2)",
				maxlen = 128,
#				 default = default_path,
				default = "")

# Export
# Frame

scene.oct_GPU_selector=BoolProperty(
				name="GPU Devices to use",
				description="Specify GPU devices to use",
				default = False)

scene.oct_GPU_use_list=StringProperty(
				name="GPUs to use",
				description="GPUs to use (ex: 0 1 2)",
				maxlen = 128,
#				 default = default_path,
				default = "")

scene.oct_animation_mode=BoolProperty(
				name="Animation mode",
				description="Animation mode",
				default = False)

scene.oct_export_mode=StringProperty(
				name="Export mode",
				description="Export mode",
				maxlen = 10,
				default = "none")
# Animation

scene.oct_image_output=StringProperty(
				name="Image Output",
				description="Image output path and name",
				maxlen = 200,
#				 default = default_path,
				default = "",
				subtype='FILE_PATH')

scene.oct_animation_start=IntProperty(attr="",
				name="Start",
				description="Start frame",
				min=1, max=20000, default=1)

scene.oct_animation_end=IntProperty(attr="",
				name="End",
				description="End frame",
				min=1, max=20000, default=1)
				
scene.oct_animation_step=IntProperty(attr="",
				name="Step",
				description="Step",
				min=1, max=100, default=1)
				
scene.oct_export_samples_per_image=IntProperty(attr="",
				name="Samples",
				description="Samples per image",
				min=4, max=64000, default=256)
				
				
scene.oct_export_samples_presets=EnumProperty(  
				name="Samples presets",
				description="Samples per image presets",
				items=(
					("0",  "Very Low (8 s/px)", ""),
					("1", "Low (32 s/px)", ""),
					("2",  "Medium (256 s/px)", ""),
					("3",  "Medium/High (512 s/px)", ""),
					("4",  "High (1024 s/px)", ""),
					("5",  "Very High (4096 s/px)", "")
				),
				default= "2")

scene.prv_oct_export_samples_presets=StringProperty(
				name="Previous Samples presets",
				description="Previous Samples presets",
				maxlen = 1,
				default = " ")
	
# Advanced (obj export)

scene.oct_export_sel_only=BoolProperty(
				name="Selection Only",
				description="Only export objects in visible selection. Else export whole scene.",
				default = False)

scene.oct_export_remove_hidden=BoolProperty(
				name="Remove Hidden",
				description="Remove hidden objects from export result",
				default = False)

scene.oct_export_apply_modifiers=BoolProperty(
				name="Apply Modifiers",
				description="Use transformed mesh data from each object. May break vert order for morph targets.",
				default = True)

scene.oct_export_ROTX90=BoolProperty(
				name="Rotate X90",
				description="Rotate on export so Blenders UP is translated into OBJs UP",
				default = True)

scene.oct_export_copy_images=BoolProperty(
				name="Copy Images",
				description="Copy image files to the export directory, never overwrite.",
				default = False)

scene.oct_export_edges=BoolProperty(
				name="Edges",
				description="Edges not connected to faces.",
				default = False)

scene.oct_export_tri=BoolProperty(
				name="Triangulate",
				description="Triangulate quads.",
				default = False)

scene.oct_export_materials=BoolProperty(
				name="Materials",
				description="Write a separate MTL file with the OBJ.",
				default = True)

scene.oct_export_UV=BoolProperty(
				name="UVs",
				description="Export texface UV coords.",
				default = True)

scene.oct_export_normals=BoolProperty(
				name="Normals",
				description="Export vertex normal data (Ignored on import).",
				default = True)

scene.oct_export_HQ=BoolProperty(
				name="HQ",
				description="Calculate high quality normals for rendering.",
				default = False)

scene.oct_export_polygroups=BoolProperty(
				name="Polygroups",
				description="Export vertex groups as OBJ groups (one group per face approximation).",
				default = False)

scene.oct_export_curves_as_nurbs=BoolProperty(
				name="Nurbs",
				description="Export 3D nurbs curves and polylines as OBJ curves, (bezier not supported).",
				default = False)


def error(error):
	raise Exception("Octane plug-in: "+error)
# 	self.update_stats("-=|ERROR|", error)
# 
# 	# Hold the render results window while we display the error message
# 	while True:
# 		if self.test_break():
# 			raise Exception("Octane plug-in: "+error)
# 		time.sleep(0.1)

def checkOctData(context):
	scene=context.scene
	
	if scene.oct_project_name=="":
		raise Exception("Octane plug-in: "+"A project name should be defined")
			
	
class octaneRender(bpy.types.RenderEngine):
	bl_idname = 'octane_RENDER'
	bl_label = "Octane Render"
	DELAY = 0.02

	def _octexport(self, scene,  basePath, baseName, frame):
		# Depends on Blender version used
		try:
			from Mathutils import Vector
		except ImportError:
			from mathutils import Vector
		from itertools import zip_longest

		ff = lambda f:format(f,'.6f').rstrip('0')

		print('OCTBP ===> Begin export')
		# Force an update to object matrices when rendering animations
		
		sdir = '{}/{}'.format(basePath,baseName)
		export_objects=[]
		objname = '{}/{}.obj'.format(basePath,baseName)
		baseNameN=baseName+'-%s' % (frame)
		baseNameP=baseName+'-%s' % (frame-1)
		objnameN = '{}/{}.obj'.format(basePath,baseNameN)
		mtlnameP = '{}/{}.mtl'.format(basePath,baseNameP)
		
		if os.path.isfile(objnameN):
			print('Previously created, skip')
			os.remove(objname)
			os.rename(objnameN,objname)
			if os.path.isfile(mtlnameP):
				os.remove(mtlnameP)
		else:
			if scene.oct_export_sel_only:
				objects = bpy.context.selected_objects
			else:
				objects = scene.objects

			for obj in objects:
				if not obj.hide_render and not obj.hide and obj.type in ['MESH', 'SURFACE', 'META', 'TEXT', 'CURVE'] and scene.layers[next((i for i in range(len(obj.layers)) if obj.layers[i]))]:
					print('OCTBP ===> Object: {}'.format(obj.name))
					export_objects.append(obj)

			write( objname, export_objects , 
					scene,scene.oct_export_tri,
					scene.oct_export_edges,
					scene.oct_export_normals,
					scene.oct_export_HQ,
					scene.oct_export_UV,
					scene.oct_export_materials,
					scene.oct_export_copy_images,
					scene.oct_export_apply_modifiers,
					scene.oct_export_ROTX90,
					True,False,True,False,
					scene.oct_export_polygroups,
					scene.oct_export_curves_as_nurbs)


	def render(self, scene):
		import subprocess
		import time
				
		if bpy.context.scene.oct_project_name=="":
			error("A project name should be defined")		
		
		print('OCTBP ===> plug-in starting...')
		self.update_stats("", "Octane: Exporting scene, please wait...")
	
		unitFactor = 1
		#Set the unit factor
		unitFactor = {
			0:0.001,
			1:0.01,
			2:0.1,
			3:1,
			4:10,
			5:100,
			6:1000,
			7:0.0254,
			8:0.3048,
			9:0.9144,
			10:201.168,
			11:1609.344               
		}[int(scene.oct_unit_size)]

		print ("Unit Factor",unitFactor)
	
		basePath = scene.oct_project_path.replace("\\", "/")
		imagePath = scene.oct_image_output.replace("\\", "/")
	
		if not os.path.exists(basePath):
			error("Cannot find scenes directory")
	
		# Get scene name
		baseName = scene.oct_project_name
		
		if os.path.isfile(scene.oct_project_name):
			baseName = os.path.basename(scene.oct_project_name)
			
		baseName = baseName.split('/')[-1].split('\\')[-1]
		
		if not baseName:
			error("Invalid scene filename")
			
		baseName = meshNode = os.path.splitext(baseName)[0].replace(" ", "_")
	
		ocsFile = '{}/{}.ocs'.format(basePath,baseName)
		objFile = '{}/{}.obj'.format(basePath,baseName)

		samplesPerImage=''
		if scene.oct_export_mode=="anim":
			samplesPerImage='-s {}'.format(scene.oct_export_samples_per_image)
			print ("Samples per image:",scene.oct_export_samples_per_image)

		while True:
			print ("Rendering frame:",scene.frame_current)
			self._octexport(scene,  basePath, baseName, scene.frame_current)
			
			if scene.oct_export_mode=="export":
				break
		
			lensAperture = ''
			if scene.oct_lens_aperture:
				lensAperture=' --cam-aperture {}'.format(scene.oct_lens_aperture_value)
				print ("Lens aperture:",scene.oct_lens_aperture_value)

# 			fdist = (scene.camera.location-scene.camera.data.dof_object.location).magnitude if scene.camera.data.dof_object else scene.camera.data.dof_distance
	
			focalDepth = ''
			if scene.oct_focal_depth:
				focalDepth=' --cam-focaldepth {} '.format(scene.oct_focal_depth_value)
				print ("Focal depth:",scene.oct_focal_depth_value)
				
			pngfile = '{}/{}_{:04d}.png'.format(imagePath,baseName,scene.frame_current)
		
			prefix=samplesPerImage+lensAperture+focalDepth
			
			camera_param = ''
			light_param=''
			
			axes = ['x','y','z']
			paramDsp=""
		
			
			if scene.oct_export_camera:
				displayError=False
				try:
					camera = bpy.data.objects[scene.oct_active_camera]
				except:
					displayError=True
				
				if scene.oct_active_camera=="" or displayError:
					error("Invalid camera object or camera not selected")
				else:
					print ("Using camera : %s" % (camera.name))
					matrix = camera.matrix_world
			
					position = rotate90x(matrix_vect(matrix,[0.0,0.0,0.0,1.0]))
					target = rotate90x(matrix_vect(matrix,[0.0,0.0,-1.0,1.0]))
					up = rotate90x(matrix_vect(matrix,[0.0,1.0,0.0,0.0]))
					for i in range(3):
						camera_param += " --cam-pos-%s %f" % (axes[i],position[i]*unitFactor)
						if i==0:paramDsp=""
						paramDsp+= "%s %f " % (axes[i],position[i]*unitFactor)
					print ("Camera position:",paramDsp)
					for i in range(3):
						camera_param += " --cam-target-%s %f" % (axes[i],target[i]*unitFactor)
						if i==0:paramDsp=""
						paramDsp+= "%s %f " % (axes[i],target[i]*unitFactor)
					print ("Camera target:",paramDsp)
					for i in range(3):
						camera_param += " --cam-up-%s %f" % (axes[i],up[i]*unitFactor)
						if i==0:paramDsp=""
						paramDsp+= "%s %f " % (axes[i],up[i]*unitFactor)
					print ("Camera up:",paramDsp)
		
					fov=camera.data.angle*180.0/3.1415926536
					camera_param += " --cam-fov %f" % fov
					print ("Camera FOV:",fov)
				
					if scene.oct_camera_motion:
						currentFrame=scene.frame_current
									    
						if scene.oct_interpolate_frame=="0":#If interpolate on next frame
							scene.frame_set(currentFrame + 1)
							print ("Motion interploate on next frame")
						elif currentFrame>1:#Interpolate on previous frame
							scene.frame_set(currentFrame - 1)
							print ("Motion interpolate on previous frame")
						
		
						matrix = scene.camera.matrix_world
			
						position = rotate90x(matrix_vect(matrix,[0.0,0.0,0.0,1.0]))
						target = rotate90x(matrix_vect(matrix,[0.0,0.0,-1.0,1.0]))
						up = rotate90x(matrix_vect(matrix,[0.0,1.0,0.0,0.0]))
			
						for i in range(3):
							camera_param += " --cam-motion-pos-%s %f" % (axes[i],position[i]*unitFactor)
							if i==0:paramDsp=""
							paramDsp+= "%s %f " % (axes[i],position[i]*unitFactor)
						print ("Camera motion position:",paramDsp)
						for i in range(3):
							camera_param += " --cam-motion-target-%s %f" % (axes[i],target[i]*unitFactor)
							if i==0:paramDsp=""
							paramDsp+= "%s %f " % (axes[i],target[i]*unitFactor)
						print ("Camera motion target:",paramDsp)
						for i in range(3):
							camera_param += " --cam-motion-up-%s %f" % (axes[i],up[i]*unitFactor)
							if i==0:paramDsp=""
							paramDsp+= "%s %f " % (axes[i],up[i]*unitFactor)
						print ("Camera motion up:",paramDsp)
							
						scene.frame_set(currentFrame)
				
			#Set the daylight option
			if scene.oct_export_sundir:
				displayError=False
				try:
					light = bpy.data.objects[scene.oct_active_light_source]
				except:
					displayError=True
				
				if scene.oct_active_light_source=="" or displayError:
					error("Invalid sun light object or sun light not selected")
				else:
					print ("Using light : %s" % (light.name))
					lightdata = light.data
					matrix = mathutils.Matrix(light.matrix_world.copy()).invert()
					matrix = rotate90x(matrix)
			
					for i in range(3):
						light_param += " --daylight-sundir-%s %f" % (axes[i],matrix[i][2]*unitFactor)
						if i==0:paramDsp=""
						paramDsp+= "%s %f " % (axes[i],matrix[i][2]*unitFactor)
					print ("Sun direction:",paramDsp)
	            
	#         octanePath=octaneRenderBinary.val
	#         octanePath='\"'+octanePath+'\"'
			
			objPath=os.path.join(scene.oct_project_path,objFile).replace("\\", "/")
		
			# Path where Octane is located
			if os.path.isdir(scene.oct_render_binary):
				octanePath = basePath+'/octane'.replace("\\", "/")
			elif os.path.isfile(scene.oct_render_binary):
				octanePath = scene.oct_render_binary.replace("\\", "/")
			else:
				error("Full path to Octane executable required")
			if not os.path.exists(octanePath):
				error("Cannot find Octane executable at specified path")
					
			projectFileAlreadyExists=False
			
			if os.path.exists(ocsFile):
					projectFileAlreadyExists=True
		
			#If linux env add quotes in file names to allow spaces
		#		if sys.platform == "linux2" or sys.platform == "darwin":
		#				objPath='\"'+objPath+'\"'
		#				objFile='\"'+objFile+'\"'
		#				ocsFile='\"'+ocsFile+'\"'
		#				pngfile='\"'+pngfile+'\"'
		
			#Set the exit after rendering flag (for animation)
			#Also set the system flag to force blender to wait Octane has finished the rendering or to immediately
			#render the control to blender (for single frame rendering)
			exitFlag=''
			commandFlag=''
			if scene.oct_export_mode=="anim":
					exitFlag=' -e -q'
					#exitFlag=' -e'
		
		#				if sys.platform == "win32":
		#						commandFlag=' /wait /normal \"\" '
		#				elif sys.platform == "linux2" or sys.platform == "darwin":
		#						commandFlag = "(nice -n %d %s)"
		#		else:
		#				if sys.platform == "win32":
		#						commandFlag=' /b /normal \"\" '
		
		#				elif sys.platform == "linux2" or sys.platform == "darwin":
		#						commandFlag = "(nice -n %d %s)&"
		
			#Set the resolution option
			resolution=''
			x=scene.oct_resolution_width
			y=scene.oct_resolution_height
			if  scene.oct_resolution:
				resSize = {
				  0: 1,
				  1: 0.75,
				  2: 0.5,
				  3: 0.25
				}[int(scene.oct_percent_size)]
				
				print ("Resolution factor :",resSize)
		
				resolution=' --film-width %i --film-height %i' % (x*resSize,y*resSize)
		
			#Set the GPUs option
			useGPU=''
			if scene.oct_GPU_selector:
				for val in scene.oct_GPU_use_list.split(' '):
						useGPU+=' -g '+val
	
			command =  octanePath+exitFlag+resolution+useGPU
			command =  exitFlag+resolution+useGPU
			
			#Use an existing octane ocs file
			if projectFileAlreadyExists and not scene.oct_replace_project:
				if scene.oct_export_mode=="anim":
					command+=' -m %s.obj -r %s %s --output-png %s %s %s %s' % (meshNode,objFile,prefix,pngfile,camera_param,light_param,ocsFile)
				else:
					command+=' -m %s.obj -r %s %s %s %s %s' % (meshNode,objFile,prefix,camera_param,light_param,ocsFile)
		
			#Force creation of the octane ocs file if it doesn't exist
			else:
				print ("Creating or replacing existing project")
					
				if scene.oct_export_mode=="anim":
					command+=' -n %s -m %s.obj -l %s %s --output-png %s %s %s' % (ocsFile,meshNode,objPath,prefix,pngfile,camera_param,light_param)
				else:
					command+=' -n %s -m %s.obj -l %s %s %s %s' % (ocsFile,meshNode,objPath,prefix,camera_param,light_param)
		
				#Reset the replace project flag to avoid accidental replacement
				#scene.oct_replace_project=False
		
		
			#If no batch file should be created -> start octane immediately
		#		else:
		#				if ostype == "win32":
		#						command='start ' + commandFlag + command
		#						print command
		#						os.system(command)
		
		#				elif ostype == "linux2" or ostype == "darwin":
		#						if exitAfter:
		#								command=commandFlag%(0, command)
		#								print command
		#								subprocess.call(command,shell=True)
		#						else:
		#								command=commandFlag%(0, command)
		#								print command
		#								os.system(command)
		
		#		print('OCTBP ===> launch Octane: {} {}/{}.ocs'.format(exepath,basePath,baseName))
			print('OCTBP ===> launch Octane: {} {} '.format(octanePath,command))
		#		octproc = subprocess.Popen([octanePath,'{}'.format(command)], cwd=basePath, shell=False)
			if sys.platform == "win32":
				octproc = subprocess.Popen('{} {}'.format(octanePath,command))
			else:
				octproc = subprocess.Popen([octanePath,'{}'.format(command)])
		
			if scene.oct_export_mode != "anim":
				break
				
			# Wait for OCT , convert and load image
			self.update_stats("", "Octane: Batch Rendering frame# {} (see console for progress), please wait...".format(scene.frame_current))
			# Hold the render results window hostage until OCT returns...
			scene.frame_set(scene.frame_current+scene.oct_animation_step)
			if scene.frame_current > scene.oct_animation_end:
				break
				
			while True:
				if octproc.poll() != None:
					result = self.begin_result(0, 0, x, y)
					try:
						print('OCTBP ===> load image from file: {}'.format(pngfile))
						result.layers[0].load_from_file('{}'.format(pngfile))
					except:
						pass

					self.end_result(result)
					break
					
				if self.test_break():
					try:
						octproc.terminate()
					except:
						pass
					break
					
				time.sleep(1)
	

# Use some of the existing buttons.
import properties_render
#properties_render.RENDER_PT_render.COMPAT_ENGINES.add('octane_RENDER')
#properties_render.RENDER_PT_dimensions.COMPAT_ENGINES.add('octane_RENDER')
#properties_render.RENDER_PT_antialiasing.COMPAT_ENGINES.add('octane_RENDER')
#properties_render.RENDER_PT_output.COMPAT_ENGINES.add('octane_RENDER')
del properties_render

# Use only a subset of the world panels
import properties_world
properties_world.WORLD_PT_preview.COMPAT_ENGINES.add('octane_RENDER')
properties_world.WORLD_PT_context_world.COMPAT_ENGINES.add('octane_RENDER')
properties_world.WORLD_PT_world.COMPAT_ENGINES.add('octane_RENDER')
properties_world.WORLD_PT_mist.COMPAT_ENGINES.add('octane_RENDER')
del properties_world

# Example of wrapping every class 'as is'
import properties_material
for member in dir(properties_material):
	subclass = getattr(properties_material, member)
	try:
		subclass.COMPAT_ENGINES.add('octane_RENDER')
	except:
		pass
del properties_material

import properties_data_mesh
for member in dir(properties_data_mesh):
    subclass = getattr(properties_data_mesh, member)
    try:
        subclass.COMPAT_ENGINES.add('octane_RENDER')
    except:
        pass
del properties_data_mesh

import properties_texture
for member in dir(properties_texture):
    subclass = getattr(properties_texture, member)
    try:
        subclass.COMPAT_ENGINES.add('octane_RENDER')
    except:
        pass
del properties_texture

import properties_data_camera
for member in dir(properties_data_camera):
    subclass = getattr(properties_data_camera, member)
    try:
        subclass.COMPAT_ENGINES.add('octane_RENDER')
    except:
        pass
del properties_data_camera

def write_settings(context):

	scene = context.scene
	
	if scene.oct_render_binary!='' or scene.oct_project_path!='':
		file = open(fixPath(os.path.join(os.path.dirname(bpy.app.binary_path),"octane.dat")), 'w')
		if scene.oct_render_binary!='':file.write("oct_render_binary:{}\n".format(scene.oct_render_binary))
		if scene.oct_project_path!='':file.write("oct_project_path:{}\n".format(scene.oct_project_path))
		file.close()	

def load_settings(context):

	scene = context.scene
	try:
		file = open(fixPath(os.path.join(os.path.dirname(bpy.app.binary_path),"octane.dat")), 'r')
		settings=file.readlines()

		for line in settings:
			keyPos = line.rfind(':')
			if keyPos == -1:
				pass
			else:
				keyName=line[:keyPos-2]
				keyVal=line[keyPos-1:-1]
				
				print ("Loading settings {}={}".format(keyName,keyVal))
				
				if keyName=="oct_render_binary":			
					scene.oct_render_binary=keyVal
				elif keyName=="oct_project_path":			
					scene.oct_project_path=keyVal
		file.close()
	except:
		pass
	

class RenderButtonsPanel():
	bl_space_type = 'PROPERTIES'
	bl_region_type = 'WINDOW'
	bl_context = "render"
	# COMPAT_ENGINES must be defined in each subclass, external engines can add themselves here

	@classmethod
	def poll(self, context):
		rd = context.scene.render
		return (rd.use_game_engine == False) and (rd.engine in self.COMPAT_ENGINES)

class button_export_objOnly(bpy.types.Operator):
	bl_idname = "button_export_objOnly"
	bl_label = "Export OBJ/MTL Only"
	bl_description = "Only export OBJ/MTL"

	def invoke(self, context, event):
		print ("Export obj only")
		scene.oct_export_mode="export"
		bpy.ops.render.render()
		scene.oct_export_mode="none"
		
		return{'FINISHED'}

class button_render(bpy.types.Operator):
	bl_idname = "button_render"
	bl_label = "Render"
	bl_description = "Export OBJ/MTL, create OCS if it doesn't exist and render"

	def invoke(self, context, event):
		print ("Render still image")
		scene.oct_export_mode="still"
		#bpy.ops.render.render(False)
		bpy.ops.render.render()
		scene.oct_export_mode="none"
		
		return{'FINISHED'}

class button_render_animation(bpy.types.Operator):
	bl_idname = "button_render_animation"
	bl_label = "Render Animation"
	bl_description = "Export OBJ/MTL and render animation frame"

	def invoke(self, context, event):
		print ("Render animation")
		# Set current frame to first octane animation frame
		currentFrame=context.scene.frame_current
		context.scene.frame_set(context.scene.oct_animation_start)

		scene.oct_export_mode="anim"
		bpy.ops.render.render()
		scene.oct_export_mode="none"

		# Restore the current frame before starting animation
		context.scene.frame_set(currentFrame)
				
		return{'FINISHED'}

class button_select_camera(bpy.types.Operator):
	bl_idname = "button_select_camera"
	bl_label = "Select"
	bl_description = "Select active camera"

	def invoke(self, context, event):
		scene = context.scene
		if scene.objects.active.type=="CAMERA":
			scene.oct_active_camera=scene.objects.active.name
		
		return{'FINISHED'}

class button_select_light_source(bpy.types.Operator):
	bl_idname = "button_select_light_source"
	bl_label = "Select"
	bl_description = "Select active light source"

	def invoke(self, context, event):
		scene = context.scene
		if scene.objects.active.type=="LAMP":
			scene.oct_active_light_source=scene.objects.active.name
		
		return{'FINISHED'}

# retrieving the masterlist with all icon names
# try:
# 	import urllib.request
# 	file = urllib.request.urlopen('https://svn.blender.org/svnroot/bf-blender/trunk/blender/source/blender/editors/include/UI_icons.h')
# except:
# 	print("Couldn't find the urllib module or was unable to connect to the internet")
# 	file = False
# if file:
# 	lines = str(file.read()).split('\\n')
# else:
# 	lines = []
# 
# # get the icon names of existing non-blank icons
# icons = []
# for l in lines:
# 	if l[:9] == 'DEF_ICON(':
# 		n = l.strip().split('(')[1].split(')')[0]
# 		if n[:10] != 'ICON_BLANK':
# 			icons.append(n[5:])
# 
# # create custom operator for each icon
# def printdoc_invoke(self, context, event):
# 	print(self.bl_description)
# 	return{'FINISHED'}
# 	
# for i in icons:
# 	c = type(i,(bpy.types.Operator,),dict(bl_idname="ICONDISPLAY_OT_"+i, bl_label="ICONDISPLAY_OT_"+i, bl_description=i))
# 	setattr(c, 'invoke', printdoc_invoke)
# 	bpy.types.register(c)
# 
# # draw the panel with the icons
# class OBJECT_PT_icons(RenderButtonsPanel,bpy.types.Panel):
# 	bl_label = "Octane Export"
# 	COMPAT_ENGINES = {'octane_RENDER'}
#     
# 	def draw(self, context):
# 		amount = 10
# 		cols = []
# 		layout = self.layout
# 		split = layout.split(percentage=1.0/amount)
# 		for i in range(amount):
# 			cols.append(split.column())
# 		for i in range(len(icons)):
# 			col = cols[i%amount].row()
# 			try:
# 				col.operator("icondisplay."+icons[i], text="", icon=icons[i])
# 			except:
# 				col.operator("icondisplay."+icons[i], text="new")

class RENDER_PT_octane_project_configuration(RenderButtonsPanel,bpy.types.Panel):
	bl_label = "Octane/OBJ Export Version 2.2"
	COMPAT_ENGINES = {'octane_RENDER'}

	def draw(self, context):
		layout = self.layout

		ocscene = context.scene	
		#rd = scene.render

		if ocscene.oct_render_binary=="":
			load_settings(bpy.context)

		row = layout.row()
		row.label(text=" Octane Project Configuration", icon='SCRIPTWIN')
		row = layout.row()
		split=row.split(percentage=0.80)
		col = split.column()
		
		
		col.prop(ocscene, "oct_render_binary")

		#if ocscene.oct_project_path=="":
		#	ocscene.oct_project_path=os.path.dirname(bpy.data.filepath)
						
		row = layout.row()
		split=row.split(percentage=0.80)
		col = split.column()
		col.prop(ocscene, "oct_project_path")

		#if scene.oct_project_name=="":
		#	ocscene.oct_project_name=os.path.splitext(os.path.basename(bpy.data.filepath))[0]
			
		write_settings(context)		

		row = layout.row()
		split=row.split(percentage=0.82)
		col = split.column()
		col.prop(ocscene, "oct_project_name")

		col = split.column()
		col.prop(ocscene, "oct_replace_project")

		row = layout.row()
		split=row.split(percentage=0.82)
		col = split.column()
		col.prop(ocscene, "oct_unit_size")
		col = split.column()


class RENDER_PT_octane_export_configuration(RenderButtonsPanel,bpy.types.Panel):
	bl_label = "Octane Export Configuration"
	COMPAT_ENGINES = {'octane_RENDER'}

	def draw(self, context):
		layout = self.layout

		ocscene = context.scene
		
		# Initialise the camera as the active camera
		#if ocscene.oct_active_camera=='':
		#	ocscene.oct_active_camera=ocscene.camera.name
			
		# Initialise the source light as the first sun light found
		if ocscene.oct_active_light_source=='':	
			for lamp in [l for l in ocscene.objects if l.type == 'LAMP']:
				if lamp.data.type=="SUN":
					ocscene.oct_active_light_source=lamp.name
					break

		# Initialise resolution
		if ocscene.oct_resolution_width==0 or ocscene.oct_resolution_height==0:
			r = ocscene.render
			ocscene.oct_resolution_width = int(r.resolution_x)
			ocscene.oct_resolution_height = int(r.resolution_y)
			

		# Film
		row = layout.row()
		row.label(text=" Film", icon='SCENE')
		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(ocscene, "oct_resolution")

		if scene.oct_resolution:
			split = split.split(percentage=0.35)
			col = split.column()
			col.prop(ocscene, "oct_resolution_width")
			split = split.split(percentage=0.6)
			col = split.column()
			col.prop(ocscene, "oct_resolution_height")
			col = split.column()
			col.prop(ocscene, "oct_percent_size")

		# Camera
		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(ocscene, "oct_export_camera")
		if ocscene.oct_export_camera:
			split = split.split(percentage=0.75)
			col = split.column()
			col.prop(ocscene, "oct_active_camera")

			col = split.column()
			col.operator("button_select_camera",icon='SCENE')

			row = layout.row()
			split = row.split(percentage=0.25)
			col = split.column()
			col.prop(ocscene, "oct_lens_aperture")
			if ocscene.oct_lens_aperture:
				split = split.split(percentage=0.75)
				col = split.column()
				col.prop(ocscene, "oct_lens_aperture_value")
				col = split.column()
				col.prop(ocscene, "oct_lens_aperture_curve")

			row = layout.row()
			split = row.split(percentage=0.25)
			col = split.column()
			col.prop(ocscene, "oct_focal_depth")
			if ocscene.oct_focal_depth:
				split = split.split(percentage=0.75)
				col = split.column()
				col.prop(ocscene, "oct_focal_depth_value")
				col = split.column()
				col.prop(ocscene, "oct_focal_depth_curve")

			row = layout.row()
			split = row.split(percentage=0.25)
			col = split.column()
			col.prop(ocscene, "oct_camera_motion")
			if scene.oct_camera_motion:
				split = split.split(percentage=0.75)
				col = split.column()
				col.prop(ocscene, "oct_interpolate_frame")
				col = split.column()
			
		# Day light environment
		layout.separator()
		row = layout.row()
		row.label(text=" Daylight Environment", icon='LAMP_SUN')
		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(ocscene, "oct_export_sundir")
		if ocscene.oct_export_sundir:
			split = split.split(percentage=0.75)
			col = split.column()
			col.prop(ocscene, "oct_active_light_source")
			col = split.column()
			col.operator("button_select_light_source", icon='LAMP_SUN')


		# GPU Devices
		row = layout.row()
		split = row.split(percentage=0.33)
		col = split.column()
		col.prop(ocscene, "oct_GPU_selector")
		if ocscene.oct_GPU_selector:
			col = split.column()
			col.prop(ocscene, "oct_GPU_use_list")

			
class RENDER_PT_octane_export(RenderButtonsPanel,bpy.types.Panel):
	bl_label = "Octane Export"
	COMPAT_ENGINES = {'octane_RENDER'}

	def draw(self, context):
		layout = self.layout

		scene = context.scene

		# Frame
		row = layout.row()
		row.label(text=" Frame", icon='SEQUENCE')
		row = layout.row()
		col = row.column()
		col.operator("button_export_objOnly",icon='SCENE_DATA')
		col = row.column()
		col.operator("button_render", icon='RENDER_STILL')
		
		# Animation
		layout.separator()
		row = layout.row()
		row.label(text=" Animation", icon='CAMERA_DATA')
		row = layout.row()

		col = row.column()
		col.prop(scene, "oct_image_output")
		
		row = layout.row()
		col = row.column()
		col.prop(scene, "oct_animation_start")
		col = row.column()
		col.prop(scene, "oct_animation_end")
		col = row.column()
		col.prop(scene, "oct_animation_step")

		row = layout.row()
		col = row.column()
		col.prop(scene, "oct_export_samples_per_image")
		col = row.column()
		col.prop(scene, "oct_export_samples_presets")
		#if scene.oct_export_samples_presets!=scene.prv_oct_export_samples_presets:
		#	scene.oct_export_samples_per_image = {
		#	  "0": 8,
		#	  "1": 32,
		#	  "2": 256,
		#	  "3": 512,
		#	  "4": 1024,
		#	  "5": 4096
		#	}[scene.oct_export_samples_presets]
			
		#	scene.prv_oct_export_samples_presets=scene.oct_export_samples_presets
			

		col = row.column()
		col.operator("button_render_animation",icon='RENDER_ANIMATION')

class RENDER_PT_octane_advanced(RenderButtonsPanel,bpy.types.Panel):
	bl_label = "Octane Advanced"
	COMPAT_ENGINES = {'octane_RENDER'}

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		
		# Context
		row = layout.row()
		row.label(text=" Context",icon='GROUP')
		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(scene, "oct_export_sel_only")
		col = split.column()
		col.prop(scene, "oct_export_remove_hidden")
		
		# Output options
		layout.separator()
		row = layout.row()
		row.label(text=" Output Options",icon='MOD_MESHDEFORM')
		row = layout.row()
		split = row.split(percentage=0.25)

		col = split.column()
		col.prop(scene, "oct_export_apply_modifiers")
		col = split.column()
		col.prop(scene, "oct_export_ROTX90")
		col = split.column()
		col.prop(scene, "oct_export_copy_images")
		col = split.column()

		layout.separator()
		row = layout.row()
		row.label(text=" Export Options",icon='SETTINGS')

		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(scene, "oct_export_edges")
		col = split.column()
		col.prop(scene, "oct_export_tri")
		col = split.column()
		col.prop(scene, "oct_export_materials")
		col = split.column()
		col.prop(scene, "oct_export_UV")

		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(scene, "oct_export_normals")
		col = split.column()
		col.prop(scene, "oct_export_HQ")
		col = split.column()
		col.prop(scene, "oct_export_polygroups")
		col = split.column()
		col.prop(scene, "oct_export_curves_as_nurbs")

def register():
	pass

def unregister():
	pass

if __name__ == "__main__":
	register()
		
